#include "ProjetdeBase.h"
#include "Dessin.h"
#include "APropos.h"
#include "Reglages.h"
#include "Calcule.h"
#include "../Icones/genpopu16.xpm"
#include "../Icones/go16.xpm"
#include "../Icones/impr16.xpm"
#include "../Icones/reglages16.xpm"
#include "../Icones/sortie16.xpm"
#include "../Icones/delete16.xpm"
#include "../Icones/fr16.xpm"
#include "../Icones/eng16.xpm"
#include "../Icones/es16.xpm"
#include "../Icones/help16.xpm"
#include <wx/file.h>
#include <wx/filename.h>
#include <wx/stdpaths.h>
#include <math.h>
#include <wx/splash.h>
#include "wx/filesys.h"
#include "wx/fs_zip.h"
#include <wx/html/helpctrl.h>

//---------------------------------------------------------------------------
//      Classe donnant l'aspect des sries affiches. Fixe un aspect par dfaut
//---------------------------------------------------------------------------
class   Tseries_aspect
        {
        private:
        public:
        std::vector<std::pair<wxString,wxColor> > aspect;
        Tseries_aspect()
                {
                PalettePerso p;
                aspect.push_back(std::make_pair(wxT("Allle 1"),p.clRed));
                aspect.push_back(std::make_pair(wxT("Allle 2"),p.clBlue));
                aspect.push_back(std::make_pair(wxT("Allle 3"),p.clGreen));
                aspect.push_back(std::make_pair(wxT("Allle 4"),p.clDkGray));
                }
        };
//---------------------------------------------------------------------------
//Variables utilises pour tests
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
// Gestion des impressions
//---------------------------------------------------------------------------
// Dclaration du Printout utilis
class GraphPrintout: public BasePrintout
{
 public:
  GraphPrintout(const wxString title = wxT("My printout"),Tgraphique* canva=NULL):BasePrintout(title),ca(canva) {}
  Tgraphique* ca;
  bool OnPrintPage(int page);
  bool HasPage(int page);
  //bool OnBeginDocument(int startPage, int endPage);
  void GetPageInfo(int *minPage, int *maxPage, int *selPageFrom, int *selPageTo);
};
//---------------------------------------------------------------------------
bool GraphPrintout::HasPage(int page)
{   // HasPage n'est pas appel avant OnPrintPage pour la premire page affiche
    // Mais est appele d'abord pour toutes les autres pages.
    return page==1;
}
//---------------------------------------------------------------------------
void GraphPrintout::GetPageInfo(int *minPage, int *maxPage, int *selPageFrom, int *selPageTo)
{
    (*minPage)=1;
    (*maxPage)=1;
}
//---------------------------------------------------------------------------
bool GraphPrintout::OnPrintPage(int page)
{
    wxDC *dc = GetDC();
    if (dc)
    {
        ca->valeurs_seulement=false;
        ca->EstImpression=true;
        ca->retrace(*dc,correctif_preview());
        ca->EstImpression=false;
        return true;
    }
    else
        return false;
}
//---------------------------------------------------------------------------
//      Fin Objet Print out pour l'impression
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// variables globales
//---------------------------------------------------------------------------
MainFrame*& firstwin()
{
    static MainFrame* fen=0;
    return fen;
}
Tparametres_mesure& parametres()
{
    static Tparametres_mesure pa;
    return pa;
}
//---------------------------------------------------------------------------
// Fentre principale
//---------------------------------------------------------------------------

//Identificateurs
const long MainFrame::idMenuQuit = wxNewId();
const long MainFrame::idMenuImprime = wxNewId();
const long MainFrame::idMenuMesure = wxNewId();
const long MainFrame::idMenuReglages = wxNewId();
const long MainFrame::idMenuStop = wxNewId();
const long MainFrame::idMenuDisclaimer = wxNewId();
const long MainFrame::idMenuAbout = wxNewId();
const long MainFrame::idMenuHelp = wxNewId();
const long MainFrame::idMenufr = wxNewId();
const long MainFrame::idMenuen = wxNewId();
const long MainFrame::idMenues = wxNewId();
const long MainFrame::idMenuToolbar1 = wxNewId();
const long MainFrame::idOnPaint = wxNewId();
const long MainFrame::idaproposinst = wxNewId();
const long MainFrame::idReglages = wxNewId();

// identifiants des contrles
const long MainFrame::idProgressBar1= wxNewId();
// identifiants des canvas
const long MainFrame::idFrequences= wxNewId();
const long MainFrame::idEffectifs= wxNewId();

//---------------------------------------------------------------------------
BEGIN_EVENT_TABLE(MainFrame, wxFrame)
    EVT_PAINT(MainFrame::OnPaint)
    EVT_CLOSE(MainFrame::OnClose)
    EVT_MENU(idMenuQuit, MainFrame::OnQuit)
    EVT_MENU(idMenuImprime, MainFrame::OnImprime)
    EVT_MENU(idMenuReglages,MainFrame::OnReglages)
    EVT_MENU(idMenuMesure,MainFrame::OnMesure)
    EVT_MENU(idMenuDisclaimer, MainFrame::OnDisclaimer)
    EVT_MENU(idMenuAbout, MainFrame::OnAbout)
    EVT_MENU(idMenuHelp, MainFrame::OnHelp)
    EVT_MENU(idMenuStop,MainFrame::OnStop)
    EVT_MENU(idMenufr,MainFrame::Onfr)
    EVT_MENU(idMenuen,MainFrame::Onen)
    EVT_MENU(idMenues,MainFrame::Ones)
END_EVENT_TABLE()
//---------------------------------------------------------------------------
// Le constructeur
MainFrame::MainFrame() : wxFrame(NULL,wxID_ANY,wxT("Genpopu"),wxPoint(800,100),wxSize(100,100),wxDEFAULT_FRAME_STYLE|wxFULL_REPAINT_ON_RESIZE)
                           ,item_label_change(true),item_menu_change(true)
//wxFULL_REPAINT_ON RESIZE est normalement le comportement par dfaut, mais semble ncessaire quand compil sous Windows
{
    Traducteur tr;
    SetTitle(wxT("Gntique des populations"));
    PalettePerso p;
    firstwin()=this;
    tr.langue_demarrage();
    SetBackgroundColour(Couleur_Fenetres_defaut());
    // Calcule la taille de dpart de la fentre
    SetSize(Taille_Fenetres_defaut());
    // Calcule la taille minimale de la fentre (ne marche pas avec Wine)
    SetSizeHints(300,200,-1,-1,-1,-1);
    FixeFontEtTaille(this);
    SetIcon(genpopu16_xpm);
    // Initialisation des chemins

    // wxStandardPaths est protected dans wxWidgets 3 donc pas instanciable
#if defined(__WXMSW__)
    InitDir=wxStandardPaths::Get().GetDataDir();
    DonneesDir=InitDir;
#elif defined(__UNIX__)
    InitDir=wxT("/usr/bin");
    DonneesDir=wxT("/usr/share/genpopu"); // Le rpertoire usr peut-tre en lecture seule
#endif
    // Graphiques
    Frequences= new Tgraphique(this,idFrequences,wxPoint(100,100),wxSize(100,100),wxT("Graphique des Frquences allliques"),wxT("Frquences allliques"),p.clLtGray,p.clYellow,p.clDkGray);
    for (unsigned i=0;i<parametres().nb_alleles();i++)
        {
        Frequences->ajoute_une_serie(wxT("Defaut"),p.clBlack);
        }
    Effectif= new Tgraphique(this,idEffectifs,wxPoint(100,100),wxSize(100,100),wxT("Graphique de l'effectif"),wxT("Effectif"),p.clLtGray,p.clYellow,p.clDkGray);
    Effectif->ajoute_une_serie(wxT("Defaut"),p.clBlack);
    srand((unsigned)time(NULL)); // initialise le gnrateur de nombres alatoires
    // Fiche Apropos
    AProposinst=new APropos(this,idaproposinst);
    // Fiche Rglages
    Reglagesinst=new Reglages(this,idReglages);
    // Aide
    wxImage::AddHandler(new wxPNGHandler());
    wxFileSystem::AddHandler(new wxZipFSHandler);
    helpController = new wxHtmlHelpController(wxHF_DEFAULT_STYLE,this);
    helpController->Initialize( DonneesDir+wxT("/")+wxT("genpopfrahlp.zip"));
    helpController->AddBook(    DonneesDir+wxT("/")+wxT("genpopenghlp.zip"));
    helpController->AddBook(    DonneesDir+wxT("/")+wxT("genpopesphlp.zip"));
#if wxCHECK_VERSION(2,9,0)
    helpController->SetShouldPreventAppExit(true);// Valeur par dfaut false provoque le plantage sous wx3.0.2 pas 2.8.2
#endif
    //---------------------------------------------------------------------------
    // Mise en place des menus
    wxMenuItem* MenuItem1;
    wxMenuItem* MenuItem2;
    wxMenuItem* MenuItem3;
    wxMenuItem* MenuItem4;
    wxMenuItem* MenuItem5;
    wxMenuItem* MenuItem6;
    wxMenuItem* MenuItem7;
    wxMenuItem* MenuItem8;
    wxMenuItem* MenuItem9;
    wxMenuItem* MenuItem10;
    wxMenuItem* MenuItem11;
    wxMenu* Menu1;
    wxMenu* Menu2;
    wxMenu* Menu3;
    wxMenu* Menu4;
    wxMenuBar* MenuBar1;
    MenuBar1 = new wxMenuBar();
    Menu1 = new wxMenu();
        MenuItem1 = new wxMenuItem(Menu1, idMenuImprime, wxT("Impression"), wxT("Imprime le diagramme"), wxITEM_NORMAL);
        Menu1->Append(MenuItem1);
        MenuItem2 = new wxMenuItem(Menu1, idMenuQuit, wxT("Quitter"), wxT("Quit the application"), wxITEM_NORMAL);
        Menu1->Append(MenuItem2);
        MenuBar1->Append(Menu1, wxT("Fichiers"));
    Menu2 = new wxMenu();
        MenuItem3 = new wxMenuItem(Menu2, idMenuMesure, wxT("Lance une mesure"), wxT("Lance une mesure"), wxITEM_RADIO);
        Menu2->Append(MenuItem3);
        Menu2->Check(idMenuMesure,false);
        MenuItem4 = new wxMenuItem(Menu1, idMenuStop, wxT("Arrt"), wxT("Arrt"), wxITEM_RADIO);
        Menu2->Append(MenuItem4);
        MenuItem5 = new wxMenuItem(Menu2, idMenuReglages, wxT("Paramtres"), wxT("Paramtres"), wxITEM_NORMAL);
        Menu2->Append(MenuItem5);
        MenuBar1->Append(Menu2, wxT("Outils"));
    Menu3 = new wxMenu();
        MenuItem6 = new wxMenuItem(Menu3, idMenufr, wxT("Franais"), wxT("Version Franaise"), wxITEM_CHECK );
        Menu3->Append(MenuItem6);
        MenuItem7 = new wxMenuItem(Menu3, idMenuen, wxT("English"), wxT("English version"), wxITEM_CHECK );
        Menu3->Append(MenuItem7);
        MenuItem8 = new wxMenuItem(Menu3, idMenues, wxT("Castellano"), wxT("Versin Castellana"), wxITEM_CHECK );
        Menu3->Append(MenuItem8);
        MenuBar1->Append(Menu3, wxT("Langage"));
    Menu4 = new wxMenu();
        MenuItem9 = new wxMenuItem(Menu4, idMenuDisclaimer, wxT("Avertissement"), wxT("Avertissement"), wxITEM_NORMAL);
        Menu4->Append(MenuItem9);
        MenuItem10 = new wxMenuItem(Menu4, idMenuAbout, wxT("A propos"), wxT("A propos"), wxITEM_NORMAL);
        Menu4->Append(MenuItem10);
        MenuItem11 = new wxMenuItem(Menu4, idMenuHelp, wxT("Aide"), wxT("Aide sur Genepopu"), wxITEM_NORMAL);
        Menu4->Append(MenuItem11);
        MenuBar1->Append(Menu4, wxT("Aide"));
    SetMenuBar(MenuBar1);
    //---------------------------------------------------------------------------
    // Barre d'outils
    wxToolBar* Toolbar1;
    Toolbar1= CreateToolBar(wxNO_BORDER | wxTB_HORIZONTAL,idMenuToolbar1);
    wxImage::AddHandler(new wxXPMHandler());
    wxBitmap* toolBarBitmaps[9];
    toolBarBitmaps[0]=new wxBitmap(go16_xpm);
    toolBarBitmaps[1]=new wxBitmap(delete16_xpm);
    toolBarBitmaps[2]=new wxBitmap(impr16_xpm);;
    toolBarBitmaps[3]=new wxBitmap(reglage16_xpm);
    toolBarBitmaps[4]=new wxBitmap(fr16_xpm);
    toolBarBitmaps[5]=new wxBitmap(eng16_xpm);
    toolBarBitmaps[6]=new wxBitmap(es16_xpm);
    toolBarBitmaps[7]=new wxBitmap(help16_xpm);
    toolBarBitmaps[8]=new wxBitmap(sortie16_xpm);

    Toolbar1->SetToolBitmapSize(wxSize(toolBarBitmaps[0]->GetWidth(),toolBarBitmaps[0]->GetHeight()));
    Toolbar1->AddTool(idMenuMesure          ,wxT("Lance une mesure"),*(toolBarBitmaps[0]),wxT("Lance une mesure"),wxITEM_RADIO);
    Toolbar1->AddTool(idMenuStop            ,wxT("Arrt")           ,*(toolBarBitmaps[1]),wxT("Arrt"),wxITEM_RADIO);
    Toolbar1->AddSeparator();
    Toolbar1->AddTool(idMenuImprime         ,wxT("Imprime")         ,*(toolBarBitmaps[2]),wxT("Imprime l'arbre")                ,wxITEM_NORMAL);
    Toolbar1->AddTool(idMenuReglages        ,wxT("Paramtres")      ,*(toolBarBitmaps[3]),wxT("Paramtres")    ,wxITEM_NORMAL);
    Toolbar1->AddSeparator();
    Toolbar1->AddTool(idMenufr              ,wxT("Franais")        ,*(toolBarBitmaps[4]),wxT("Franais")      ,wxITEM_RADIO);
    Toolbar1->AddTool(idMenuen              ,wxT("English")         ,*(toolBarBitmaps[5]),wxT("English")       ,wxITEM_RADIO);
    Toolbar1->AddTool(idMenues              ,wxT("espaol")         ,*(toolBarBitmaps[6]),wxT("espaol")       ,wxITEM_RADIO);
    Toolbar1->AddSeparator();
    Toolbar1->AddTool(idMenuHelp            ,wxT("Aide")            ,*(toolBarBitmaps[7]),wxT("Aide")          ,wxITEM_NORMAL);
    Toolbar1->AddSeparator();
    Toolbar1->AddTool(idMenuQuit            ,wxT("Quitter")         ,*(toolBarBitmaps[8]),wxT("Sortie")        ,wxITEM_NORMAL);
    Toolbar1->Realize();
    SetToolBar(Toolbar1);
    //---------------------------------------------------------------------------
    // Labels
    Label1=new wxStaticText(this,wxID_ANY,wxT("<-- panmixie -->"),wxPoint(10,10),wxSize(20,100),wxST_NO_AUTORESIZE);
    Label1a=new wxStaticText(this,wxID_ANY,wxT("Echap pour interrompre"),wxPoint(10,10),wxSize(20,100),wxST_NO_AUTORESIZE);
    Label1a->Enable(false);
    Label2=new wxStaticText(this,wxID_ANY,wxT("Effectif Mini"),wxPoint(10,10),wxSize(20,100),wxST_NO_AUTORESIZE);
    Label2a=new wxStaticText(this,wxID_ANY,wxT("valnum"),wxPoint(10,10),wxSize(20,100),wxST_NO_AUTORESIZE);
    Label3=new wxStaticText(this,wxID_ANY,wxT("Effectif Maxi"),wxPoint(10,10),wxSize(20,100),wxST_NO_AUTORESIZE);
    Label3a=new wxStaticText(this,wxID_ANY,wxT("valnum"),wxPoint(10,10),wxSize(20,100),wxST_NO_AUTORESIZE);
    Label4=new wxStaticText(this,wxID_ANY,wxT("Nb gnrations"),wxPoint(10,10),wxSize(20,100),wxST_NO_AUTORESIZE);
    Label4a=new wxStaticText(this,wxID_ANY,wxT("valnum"),wxPoint(10,10),wxSize(20,100),wxST_NO_AUTORESIZE);
    Label5=new wxStaticText(this,wxID_ANY,wxT("Fcondit"),wxPoint(10,10),wxSize(20,100),wxST_NO_AUTORESIZE);
    Label5a=new wxStaticText(this,wxID_ANY,wxT("valnum"),wxPoint(10,10),wxSize(20,100),wxST_NO_AUTORESIZE);

    // Contrles
    ProgressBar1=new wxGauge(this,idProgressBar1,100,wxPoint(10,10),wxSize(10,10),wxGA_HORIZONTAL);
    g_printData =new wxPrintData;
    g_printData->SetPaperId(wxPAPER_A4);
}
//---------------------------------------------------------------------------
// Le Destructeur
MainFrame::~MainFrame()
{
    delete g_printData;
}
//---------------------------------------------------------------------------
void MainFrame::OnImprime(wxCommandEvent& event)
{
    Traducteur tr;
    wxPrintDialogData g_printDialogData(*g_printData);
    g_printDialogData.EnablePageNumbers(false);
    //g_printDialogData.SetMaxPage(3);
    wxPrintPreview *preview = new wxPrintPreview(new GraphPrintout(tr.trad(wxT("Prvisualisation")),Frequences), new GraphPrintout(tr.trad(wxT("Impression")),Frequences),& g_printDialogData);
    if (!preview->Ok())
    {
        delete preview;
        wxMessageBox(wxT("There was a problem previewing.\nPerhaps your current printer is not set correctly?"), wxT("Previewing"), wxOK);
        return;
    }
    wxPreviewFrame *frame = new wxPreviewFrame(preview, this, tr.trad(wxT("Gntique des populations")), wxPoint(100, 100), wxSize(600, 650));
    frame->Centre(wxBOTH);
    frame->Initialize();
    frame->Show();
}
//---------------------------------------------------------------------------
void MainFrame::OnQuit(wxCommandEvent& event)
{
    wxCloseEvent eventclose;
    OnClose(eventclose);
}
//---------------------------------------------------------------------------
void MainFrame::OnClose(wxCloseEvent& event)
{
    this->Destroy();
}
//---------------------------------------------------------------------------
void MainFrame::OnMesure(wxCommandEvent& event)
{
    // Lance un calcul

// valtest=Calculs::teste_RandomP(parametres().fecondite); // faire sauter
    Reglagesinst->Enable(false);
    Label1a->Enable(true);// message indiquant que la touche Echap est active
    Frequences->purge_valeurs();
    Effectif->purge_valeurs();
    Frequences->valeurs_seulement=true;
    Effectif->valeurs_seulement=true;
    calcul_en_cours=true;
    item_menu_change=true;
    Refresh();
    Update();// force l'affichage correct des diffrents boutons.
    Calculs::Mesure();
    Frequences->valeurs_seulement=false;
    Effectif->valeurs_seulement=false;
    Label1a->Enable(false);
    Reglagesinst->Enable(true);
    calcul_en_cours=false;
    item_menu_change=true;
    Refresh();
    Update();// force l'affichage correct des diffrents boutons.
}
//---------------------------------------------------------------------------
void MainFrame::OnReglages(wxCommandEvent& event)
{
    Reglagesinst->Show();
    Reglagesinst->Update();
}
//---------------------------------------------------------------------------
void MainFrame::OnStop(wxCommandEvent& event)
{
    calcul_en_cours=false;
    item_menu_change=true;
    rafraichit();
}
//---------------------------------------------------------------------------
void MainFrame::OnDisclaimer(wxCommandEvent& event)
{
    Traducteur tr;
    wxBitmap bitmap;
    wxImage::AddHandler(new wxPNGHandler());
    if (bitmap.LoadFile(DonneesDir+wxT("/")+tr.trad(wxT("DisclaimimageFra.png")), wxBITMAP_TYPE_PNG))
    {
      wxSplashScreen* splash __attribute__ ((unused));
      splash = new wxSplashScreen(bitmap,
          wxSPLASH_CENTRE_ON_SCREEN|wxSPLASH_TIMEOUT,
          6000, NULL, -1, wxDefaultPosition, wxDefaultSize,
          wxSIMPLE_BORDER|wxSTAY_ON_TOP);
    }
    wxYield();
}
//---------------------------------------------------------------------------
void MainFrame::OnAbout(wxCommandEvent& event)
{
    AProposinst->Show();
}
//---------------------------------------------------------------------------
 void MainFrame::OnHelp(wxCommandEvent& event)
{
        Traducteur tr;
        helpController->DisplayContents();
        helpController->Display(tr.trad(wxT("genpopfrahlp")));
}
//---------------------------------------------------------------------------
void MainFrame::Onfr(wxCommandEvent& event)
{
    Traducteur tr;
    tr.fixe_langue()=fr;
    item_menu_change=true;
    Refresh();
}
void MainFrame::Onen(wxCommandEvent& event)
{
    Traducteur tr;
    tr.fixe_langue()=ang;
    item_menu_change=true;
    Refresh();
}
void MainFrame::Ones(wxCommandEvent& event)
{
    Traducteur tr;
    tr.fixe_langue()=esp;
    item_menu_change=true;
    Refresh();
}
//---------------------------------------------------------------------------
void MainFrame::OnPaint(wxPaintEvent& event)
{
    wxPaintDC dc(this);
    rafraichit();
}
//---------------------------------------------------------------------------
void MainFrame::rafraichit()
{
    Traducteur tr;
    PalettePerso p;
    if (item_menu_change)
    {
        item_menu_change=false;
        wxMenuBar* Menuprov=GetMenuBar();
        // Traduit les titres des enfants
        RenommeEnfants(this);
        // Titres de menus
        size_t nb_menus=Menuprov->GetMenuCount();
        for (size_t i=0;i<nb_menus;i++)
            Menuprov->SetMenuLabel(i,tr.trad_tout(Menuprov->GetMenuLabel(i)));
        //Items de chaque menu
        Menuprov->SetLabel(idMenuQuit,tr.trad(wxT("Quitter")));
        Menuprov->SetLabel(idMenuImprime,tr.trad(wxT("Impression")));
        Menuprov->SetLabel(idMenuMesure,tr.trad(wxT("Lance une mesure")));
        Menuprov->SetLabel(idMenuStop,tr.trad(wxT("Arrt")));
        Menuprov->SetLabel(idMenuReglages,tr.trad(wxT("Paramtres")));
        Menuprov->SetLabel(idMenuDisclaimer, tr.trad(wxT("Avertissement")));
        Menuprov->SetLabel(idMenuAbout, tr.trad(wxT("A propos")));
        Menuprov->SetLabel(idMenuHelp, tr.trad(wxT("Aide")));
        Menuprov->SetLabel(idMenufr,tr.trad(wxT("Franais")));
        Menuprov->SetLabel(idMenuen,tr.trad(wxT("English")));
        Menuprov->SetLabel(idMenues,tr.trad(wxT("Castellano")));
         // titre les axes des graphiques
        Effectif->change_aspect_serie(tr.trad(wxT("Effectif")),p.clGreen,0);
        Effectif->caracteristiques_axeY.titre=tr.trad(wxT("Effectif"));
        Effectif->caracteristiques_axeY.unite=tr.trad(wxT("Nb individus"));
        Effectif->caracteristiques_axeY.val_max_affichee=parametres().effectif_max;
        Effectif->caracteristiques_axeX.titre=tr.trad(wxT("Nb gnrations"));
        Effectif->caracteristiques_axeX.unite=wxString::Format(wxT("%5i"),Effectif->caracteristiques_axeX.valeur_de_la_graduation())+tr.trad(wxT(" par graduation"));
        Frequences->caracteristiques_axeY.titre=tr.trad(wxT("Frquences allliques"));
        Frequences->caracteristiques_axeY.unite=tr.trad(wxT("%"));
        Frequences->caracteristiques_axeX.titre=tr.trad(wxT("Nb gnrations"));
        Frequences->caracteristiques_axeX.unite=wxString::Format(wxT("%5i"),Effectif->caracteristiques_axeX.valeur_de_la_graduation())+tr.trad(wxT(" par graduation"));
        Effectif->Refresh();
        Frequences->Refresh();
        Effectif->SetFocus(); //Ncessaire sous Linux pour que la touche Echap soit capture car ... ?
        // Boutons actifs ou non sur la barre d'outils
        langues langue_en_cours=tr.fixe_langue();
        Menuprov->Check(idMenufr,(langue_en_cours==fr));
        Menuprov->Check(idMenuen,(langue_en_cours==ang));
        Menuprov->Check(idMenues,(langue_en_cours==esp));
#if defined(__WXMSW__)
        Menuprov->Check(idMenuMesure,(calcul_en_cours));
        Menuprov->Check(idMenuStop,(!calcul_en_cours));// Sous Linux semble provoquer l'mission de l'vnement correspondant
#elif defined(__UNIX__)
        // Sous Linux Check semble provoquer l'mission de l'vnement correspondant
#endif
        wxToolBar* Toolbarprov=GetToolBar();

        Toolbarprov->ToggleTool(idMenuMesure,calcul_en_cours);
        Toolbarprov->ToggleTool(idMenuStop,!calcul_en_cours);
        Toolbarprov->ToggleTool(idMenufr,langue_en_cours==fr);
        Toolbarprov->ToggleTool(idMenuen,langue_en_cours==ang);
        Toolbarprov->ToggleTool(idMenues,langue_en_cours==esp);
        // traduit les bulles d'aide des boutons
        Toolbarprov->SetToolShortHelp(idMenuMesure,tr.trad(wxT("Lance une mesure")));
        Toolbarprov->SetToolShortHelp(idMenuStop,tr.trad(wxT("Arrt")));
        Toolbarprov->SetToolShortHelp(idMenuImprime,tr.trad(wxT("Impression")));
        Toolbarprov->SetToolShortHelp(idMenuReglages,tr.trad(wxT("Paramtres")));
        Toolbarprov->SetToolShortHelp(idMenuHelp, tr.trad(wxT("Aide")));
        Toolbarprov->SetToolShortHelp(idMenuQuit,tr.trad(wxT("Quitter")));
    }
    // Ajuste la taille de la zone graphique
    Ajuste_taille_et_pos(wxPoint(280,50),wxSize(710,550),this,Frequences);
    Ajuste_taille_et_pos(wxPoint(280,610),wxSize(710,380),this,Effectif);
    // Ajuste la taille des contrles
    Ajuste_taille_et_pos(wxPoint(5,50),wxSize(275,20),this,ProgressBar1);
    // Ajuste la position des labels
    int labx(10),ecart(150),laby(200),pasy(100);
    Ajuste_taille_et_pos(wxPoint(80,100),wxSize(150,50),this,Label1);
    Ajuste_taille_et_pos(wxPoint(labx,laby),wxSize(150,50),this,Label2); // effectif min
    Ajuste_taille_et_pos(wxPoint(labx+ecart,laby),wxSize(100,50),this,Label2a);
    Ajuste_taille_et_pos(wxPoint(labx,laby+pasy),wxSize(150,50),this,Label3); // Effectif max
    Ajuste_taille_et_pos(wxPoint(labx+ecart,laby+pasy),wxSize(100,50),this,Label3a);
    Ajuste_taille_et_pos(wxPoint(labx,laby+2*pasy),wxSize(150,50),this,Label4); // nb gnrations
    Ajuste_taille_et_pos(wxPoint(labx+ecart,laby+2*pasy),wxSize(100,50),this,Label4a);
    Ajuste_taille_et_pos(wxPoint(labx,laby+3*pasy),wxSize(150,50),this,Label5); // Fcondit
    Ajuste_taille_et_pos(wxPoint(labx+ecart,laby+3*pasy),wxSize(100,50),this,Label5a);
    Ajuste_taille_et_pos(wxPoint(labx,laby+4*pasy),wxSize(200,50),this,Label1a);
    // Mise  jour des champs de valeurs si un rglage a t effectu
    if(item_label_change)
        {
        item_label_change=false;
        Label2a->SetLabel(wxString::Format(wxT("%06i"),parametres().effectif_depart));
        Label3a->SetLabel(wxString::Format(wxT("%06i"),parametres().effectif_max));
        Label4a->SetLabel(wxString::Format(wxT("%06i"),parametres().nb_generations));
        Label5a->SetLabel(wxString::Format(wxT("%04.2f"),parametres().fecondite));
        // Met  jour les chelles de l'ordonne du graphe des effectifs
        Effectif->caracteristiques_axeY.valeur_de_la_graduation()=parametres().effectif_max/10;
        Effectif->caracteristiques_axeY.val_max_affichee=parametres().effectif_max;
        // Au cas o ce serait l'chelle des abscisses serait modifie
        Effectif->caracteristiques_axeX.unite=wxString::Format(wxT("%5i"),Effectif->caracteristiques_axeX.valeur_de_la_graduation())+tr.trad(wxT(" par graduation"));
        Frequences->caracteristiques_axeX.unite=wxString::Format(wxT("%5i"),Effectif->caracteristiques_axeX.valeur_de_la_graduation())+tr.trad(wxT(" par graduation"));
        }
    Tseries_aspect asp;
    // Traite le cas d'un changement de nombre d'allles
    if (parametres().nb_alleles()!=Frequences->nombre_series())
            {Frequences->supprime_toutes_les_series();
            Effectif->purge_valeurs();
            }
    // renomme ou rajoute (si absentes) les diffrentes sries
    for (unsigned i=0;i<parametres().nb_alleles();i++)
            {wxString Titre=tr.trad(asp.aspect[i].first);
            wxColor Couleur=asp.aspect[i].second;
            if (!Frequences->change_aspect_serie(Titre,Couleur,i))
                    Frequences->ajoute_une_serie(Titre,Couleur);
            }
    // traite le cas d'un changement de nombre de gnrations
    if (parametres().nb_generations!=unsigned(Effectif->caracteristiques_axeX.val_max_affichee))
            {Effectif->caracteristiques_axeX.val_max_affichee=parametres().nb_generations;
            Frequences->caracteristiques_axeX.val_max_affichee=parametres().nb_generations;
            }
    // Rafrachit les autres fentres
    if (Reglagesinst->IsShown()) Reglagesinst->Refresh();
}
//---------------------------------------------------------------------------
//      Dfinitions de la classe Tparametres_mesure
//---------------------------------------------------------------------------
        Tparametres_mesure::Tparametres_mesure():
                nb_all(2),
                effectif_depart(300),effectif_max(2000),
                nb_generations(100),
                fecondite(2.1),
                modification_autorisee(true)
        {nb_alleles();}
//---------------------------------------------------------------------------
        // Dfinition des constantes dclares dans l'en tte
        const unsigned Tparametres_mesure::facteur_de_precision;
        const unsigned Tparametres_mesure::nb_max_enfants;
        const unsigned Tparametres_mesure::limite_inf_nb_generations;
        const unsigned Tparametres_mesure::limite_sup_nb_generations;
        const unsigned Tparametres_mesure::limite_inf_effectif;
        const unsigned Tparametres_mesure::limite_sup_effectif;
        const float Tparametres_mesure::limite_inf_fecondite;
        const float Tparametres_mesure::limite_sup_fecondite;
//---------------------------------------------------------------------------
        unsigned& Tparametres_mesure::nb_alleles()
        // Fonctionne comme un alias pour nb_all
        {//maintien la cohrence entre le nombre d'allles et les frquences de dpart et la ltalit
        if (nb_all !=frequences_alleliques.size())
                {frequences_alleliques.clear();
                valeur_letalite.clear();
                for (unsigned i=0;i<nb_all;i++)
                        {frequences_alleliques.push_back(1.0/nb_all);
                        valeur_letalite.push_back(0);
                        }
                }
        return nb_all;
        }
//---------------------------------------------------------------------------
        bool Tparametres_mesure::modifie_frequence(unsigned numal,float valeur)
        {bool possible=(valeur>=0)&&(valeur<=1)&&(numal<=nb_all);
        if (possible)
                {float total=0.0001;
                float ecart=frequences_alleliques[numal]-valeur;
                for (unsigned i=0;i<nb_all;i++)
                        if (i!=numal) total+=frequences_alleliques[i];
                for (unsigned i=0;i<nb_all;i++)
                        if (i!=numal) frequences_alleliques[i]*=(total+ecart)/total;
                frequences_alleliques[numal]=valeur;
                }
        return possible;
        }
//---------------------------------------------------------------------------
        // Simple accs contrl au conteneur frquences_alleliques
        float Tparametres_mesure::frequence(unsigned numal)
        {float resultat=0;
        if (numal<=nb_all) resultat=frequences_alleliques[numal];
        return resultat;
        }
//---------------------------------------------------------------------------
        bool Tparametres_mesure::modifie_letalite(unsigned numal,float valeur)
        {bool possible=(valeur>=0)&&(valeur<=1)&&(numal<=nb_all);
        if (possible) valeur_letalite[numal]=valeur;
        return possible;
        }
//---------------------------------------------------------------------------
        // Simple accs contrl au conteneur valeur_letalite
        float Tparametres_mesure::letalite(unsigned numal)
        {float resultat=0;
        if (numal<=nb_all) resultat=valeur_letalite[numal];
        return resultat;
        }
//---------------------------------------------------------------------------
        bool Tparametres_mesure::operator==(Tparametres_mesure autre_mesure)
        {bool est_egal(false);
        est_egal=(effectif_depart==autre_mesure.effectif_depart);
        est_egal&=(effectif_max==autre_mesure.effectif_max);
        est_egal&=(nb_generations==autre_mesure.nb_generations);
        est_egal&=(fecondite==autre_mesure.fecondite);
        est_egal&=(nb_all==autre_mesure.nb_all);
        for (unsigned int i=0;i<nb_all;i++)
                {
                est_egal&=(frequences_alleliques[i]==autre_mesure.frequences_alleliques[i]);
                est_egal&=(valeur_letalite[i]==autre_mesure.valeur_letalite[i]);
                }
        return est_egal;
        }
//---------------------------------------------------------------------------
        bool Tparametres_mesure::valeur_modifiee()
        {bool a_change(false);
        static Tparametres_mesure ancienne_valeur;
        a_change= !(*this==ancienne_valeur);
        if (a_change) ancienne_valeur=(*this);
        return a_change;
        }
//---------------------------------------------------------------------------
//      Fin de l'objet Tparametres_mesure
//---------------------------------------------------------------------------
